<?php
namespace FlyingProxy;

class Purge {
  public static function init() {
    if (!get_option('flyingproxy_api_key')) {
      return;
    }

    // Purge necessary URLs when a post is published/updated/deleted
    add_action('save_post', ['FlyingProxy\Purge', 'post_saved'], 10, 2);

    // Purge post URL when comment is published
    add_action('comment_post', ['FlyingProxy\Purge', 'purge_on_comment'], 10, 2);

    // Purge product page when stock is updated in WooCommerce
    add_action('woocommerce_product_set_stock', ['FlyingProxy\Purge', 'purge_on_stock_change']);
    add_action('woocommerce_variation_set_stock', ['FlyingProxy\Purge', 'purge_on_stock_change']);

    // Invalidate cache on common actions like plugin activation/deactivation, theme change etc
    add_action('shutdown', ['FlyingProxy\Purge', 'register_invalidate_actions']);
  }

  public static function post_saved($post_id, $post) {
    $post_url = get_permalink($post_id);

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return;
    }

    if (wp_is_post_autosave($post_id) || wp_is_post_revision($post_id)) {
      return;
    }

    $allowed_statuses = apply_filters('flyingproxy_purge_post_statuses', ['publish']);
    if (!in_array($post->post_status, $allowed_statuses)) {
      return;
    }

    if ($post->post_type == 'page') {
      self::purge_by_urls([$post_url]);
      return;
    }

    // URLs to purge
    $urls = [];

    // Add post URL
    if ($post->post_status === 'publish') {
      $urls[] = $post_url;
    }

    // Add home URL
    $urls[] = home_url();

    // Add blog URL
    $urls[] = get_bloginfo('url');

    // Add category URLs
    $categories = get_the_category($post_id);
    foreach ($categories as $category) {
      $urls[] = get_category_link($category->term_id);
    }

    // Add WooCommerce product category URLs if WooCommerce is active
    if (class_exists('WooCommerce')) {
      $product_categories = get_the_terms($post_id, 'product_cat');
      if ($product_categories) {
        foreach ($product_categories as $product_category) {
          $urls[] = get_term_link($product_category->term_id);
        }
      }
    }

    // Add tag URLs (if any)
    $tags = get_the_tags($post_id);
    if ($tags) {
      foreach ($tags as $tag) {
        $urls[] = get_tag_link($tag->term_id);
      }
    }

    // Add author URL
    $urls[] = get_author_posts_url($post->post_author);

    // Add WooCommerce shop URL if it's a product
    if (class_exists('woocommerce') && $post->post_type == 'product') {
      $urls[] = get_permalink(wc_get_page_id('shop'));
    }

    // Find unique URLs
    $urls = array_unique($urls);

    // Purge URLs
    self::purge_by_urls($urls);
  }

  public static function register_invalidate_actions() {
    $actions = [
      'activate_plugin',
      'deactivate_plugin',
      'switch_theme',
      'customize_save',
      'update_option_permalink_structure',
      'update_option_tag_base',
      'update_option_category_base',
      'update_option_WPLANG',
      'update_option_blogname',
      'update_option_blogdescription',
      'update_option_blog_public',
      'update_option_show_on_front',
      'update_option_page_on_front',
      'update_option_page_for_posts',
      'update_option_posts_per_page',
      'update_option_woocommerce_permalinks',
      'automatic_updates_complete',
      '_core_updated_successfully',
    ];

    foreach ($actions as $action) {
      if (did_action($action)) {
        self::invalidate();
        break;
      }
    }
  }

  public static function purge_on_comment($comment_id, $comment_approved) {
    if (!$comment_approved) {
      return;
    }
    $comment = get_comment($comment_id);
    $post_id = $comment->comment_post_ID;
    $url = get_permalink($post_id);
    self::purge_by_urls([$url]);
  }

  public static function purge_on_stock_change($product) {
    // Check product stock quantity
    $stock_quantity = $product->get_stock_quantity();

    // Skip if stock quantity is not zero
    if ($stock_quantity) {
      return;
    }

    $url = get_permalink($product->get_id());
    self::purge_by_urls([$url]);
  }

  public static function purge_by_urls($urls) {
    wp_remote_post(FLYINGPROXY_API_URL . '/purge/urls', [
      'blocking' => false,
      'sslverify' => false,
      'headers' => [
        'Content-Type' => 'application/json',
        'x-api-key' => get_option('flyingproxy_api_key'),
      ],
      'body' => json_encode(array_values($urls)),
    ]);
  }

  public static function invalidate() {
    wp_remote_post(FLYINGPROXY_API_URL . '/purge/invalidate', [
      'blocking' => false,
      'sslverify' => false,
      'headers' => [
        'Content-Type' => 'application/json',
        'x-api-key' => get_option('flyingproxy_api_key'),
      ],
    ]);
  }
}